<?php
/* Copyright (C) 2019      Open-DSI             <support@open-dsi.fr>
 * Copyright (C) 2023      Frédéric France      <frederic.france@netlogic.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file    htdocs/sirene/class/sirene.class.php
 * \ingroup sirene
 * \brief
 */

// check if we can autoload guzzle (due to other autoloader)
if (!class_exists('\GuzzleHttp\Client')) {
    dol_include_once('/sirene/vendor/autoload.php');
}


use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;

/**
 * Class Sirene
 *
 * Put here description of your class
 */
class Sirene
{
    const COMPANY_ADMIN_STATUS_CLOSED = 'F';
    const COMPANY_ADMIN_STATUS_OPENED = 'A';

    /**
     * @var DoliDB Database handler.
     */
    public $db;
    /**
     * @var string Error
     */
    public $error = '';
    /**
     * @var array Errors
     */
    public $errors = array();

    /**
     * @var Client      Client REST handler
     */
    public $client;

    /**
     * @var array Result of the request to get companies with the Sirene API
     */
    public $companies_results = array();

    /**
     * @var array Result of the request to get a single company with the Siren API
     */
    public $single_company_result = array();


    /**
     * Constructor
     *
     * @param DoliDB $db Database handler
     */
    public function __construct($db)
    {
        $this->db = $db;
    }

    /**
     * get authorization header
     * depending on API version and date
     * @return array
     */
    private function _getAuthorizationHeader()
    {
        global $conf;

        return ['X-INSEE-Api-Key-Integration' => $conf->global->SIRENE_API_BEARER_KEY ?? ''];
    }

    /**
     * Get API URL depending on configuration
     * 
     * @return string    the API url
     */
    public static function getSireneApiUrl()
    {
        return 'https://api.insee.fr/api-sirene/3.11/';
    }

    /**
     * Get the date after which an API update is required.
     */
    public static function datetimeChangeApiAddress()
    {
        return dol_mktime(0, 0, 0, 02, 28, 2025);
    }

    /**
     *  conditionDate
     *
     * @param string $Field Field operand 1
     * @param string $Value Value operand 2
     * @param string $Sens Comparison operator
     * @return string
     */
    public function conditionDate($Field, $Value, $Sens)
    {
        // TODO date_format is forbidden, not performant and not portable. Use instead BETWEEN
        if (strlen($Value) == 4) {
            $Condition = " date_format(".$Field.",'%Y') ".$Sens." '".$Value."'";
        } elseif (strlen($Value) == 6) {
            $Condition = " date_format(".$Field.",'%Y%m') ".$Sens." '".$Value."'";
        } else {
            $Condition = " date_format(".$Field.",'%Y%m%d') ".$Sens." ".$Value;
        }
        return $Condition;
    }

    /**
     * @return int
     * @throws Exception
     */
    public function cronSirene()
    {
        global $user, $langs, $sirene, $conf;
        $langs->load('sirene@sirene');
        $error = 0;
        $output = '';

        // Get all companies to check with sirene
        $companies_to_check = array();
        $sql = "SELECT s.rowid";
        $sql .= ", " . $this->db->ifsql("cte.id IS NOT NULL AND cte.code = 'TE_PRIVATE'", 1, 0) . " as private";
        $sql .= ", s.nom as name, s.name_alias, s.address, s.zip, s.town";
        $sql .= ", s.fk_departement as state_id, s.fk_pays as country_id, s.siren as idprof1";
        $sql .= ", s.siret as idprof2, s.ape as idprof3, s.idprof6, s.tva_intra";
        $sql .= " FROM " . MAIN_DB_PREFIX . "societe AS s";
        $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "societe_extrafields AS sef ON sef.fk_object = s.rowid";
        $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "c_typent AS cte ON cte.id = s.fk_typent";
        $sql .= " WHERE s.status = 1";
        $sql .= " AND (sef.sirene_status = 0 OR sef.sirene_status IS NULL)";
        $sql .= " AND s.siret != ''";
        $sql .= " ORDER BY rowid ASC";

        $resql = $this->db->query($sql);
        if (!$resql) {
            $this->error = 'Error ' . $this->db->lasterror();
            $this->errors = array();
            dol_syslog(__METHOD__ . " SQL: " . $sql . "; Error: " . $this->db->lasterror(), LOG_ERR);
            return -1;
        } else {
            while ($obj = $this->db->fetch_object($resql)) {
                $siret = str_replace(' ', '', trim($obj->idprof2));
                if (empty($obj->idprof2)) continue;
                $obj->idprof2 = $siret;

                if (isset($companies_to_check[$siret])) {
                    $error_msg = "Warning: Same siret {$obj->idprof2} for company {$companies_to_check[$siret]->name} (ID : {$companies_to_check[$siret]->rowid}) and company {$obj->name} (ID : {$obj->rowid})";
                    dol_syslog(__METHOD__ . " " . $error_msg, LOG_WARNING);
                    $output .= '<span style="color: orangered;"> ' . $error_msg . '</span>' . "<br>";
                } else {
                    $companies_to_check[$siret] = $obj;
                }
            }
            $this->db->free($resql);
        }

        if (!empty($companies_to_check)) {
            require_once DOL_DOCUMENT_ROOT . '/societe/class/societe.class.php';

            $nb_request_by_group = !empty($conf->global->SIRENE_NB_REQUEST_BY_GROUP) ? $conf->global->SIRENE_NB_REQUEST_BY_GROUP : 100;
            $nb_request_by_time_limit = !empty($conf->global->SIRENE_NB_REQUEST_BY_TIME_LIMIT) ? $conf->global->SIRENE_NB_REQUEST_BY_TIME_LIMIT : 20;
            $time_limit_all_request = !empty($conf->global->SIRENE_TIME_LIMIT_ALL_REQUEST) ? $conf->global->SIRENE_TIME_LIMIT_ALL_REQUEST : 60;
            // not more than 20 requests by minute
            if ($nb_request_by_time_limit > 20) {
                $nb_request_by_time_limit = 20;
            }
            // not less than 1 minute by request
            if ($time_limit_all_request < 60) {
                $time_limit_all_request = 60;
            }

            $to_update = array(
                'name'                  => !empty($conf->global->SIRENE_CRON_COMPANY_NAME),
                'name_alias'            => !empty($conf->global->SIRENE_CRON_COMPANY_NAME_ALIAS),
                'address'               => !empty($conf->global->SIRENE_CRON_ADRESS),
                'zip'                   => !empty($conf->global->SIRENE_CRON_COMPANY_ZIP),
                'town'                  => !empty($conf->global->SIRENE_CRON_CITY),
                'state_id'              => !empty($conf->global->SIRENE_CRON_STATE_CODE),
                'country_id'            => !empty($conf->global->SIRENE_CRON_COUNTRY),
                'idprof1'               => !empty($conf->global->SIRENE_CRON_SIREN),
                'idprof2'               => !empty($conf->global->SIRENE_CRON_SIRET),
                'idprof3'               => !empty($conf->global->SIRENE_CRON_NAF),
                'idprof6'               => !empty($conf->global->SIRENE_CRON_RNA),
                'tva_intra'             => !empty($conf->global->SIRENE_CRON_TVA),
                'effectif_id'           => !empty($conf->global->SIRENE_CRON_STAFF),
                'forme_juridique_code'  => !empty($conf->global->SIRENE_CRON_JURI_STATUS)
            );
            $correspondance = array(
                // 'private' => 'private',
                'company_name' => 'name',
                'company_name_alias' => 'name_alias',
                'address' => 'address',
                'zipcode' => 'zip',
                'town' => 'town',
                'state_id' => 'state_id',
                'country_id' => 'country_id',
                'siren' => 'idprof1',
                'siret' => 'idprof2',
                'codenaf_san' => 'idprof3',
                'rna' => 'idprof6',
                'sirene_tva_intra' => 'tva_intra',
                // 'staff' => 'effectif', //effectif
                'staff' => 'effectif_id', //effectif
                // 'legalcategory_level_2' => 'forme_juridique',  //type d'entité légale
                'legalcategory' => 'forme_juridique_code',  //type d'entité légale
            );
            $grouped_companies_to_check = array();
            $current_group = array();
            $count = 0;
            foreach ($companies_to_check as $k => $v) {
                $current_group[$k] = $v;

                $count++;
                if ($count >= $nb_request_by_group) {
                    $count = 0;
                    $grouped_companies_to_check[] = $current_group;
                    $current_group = array();
                }
            }
            if (!empty($current_group)) {
                $grouped_companies_to_check[] = $current_group;
            }

            $sirene = new self($this->db);

            // Connection to sirene API
            $result = $sirene->connectionSireneApi();
            if ($result < 0) {
                $this->error = $langs->trans('SireneErrorWhileConnect') . ': ' . $sirene->errorsToString();
                $this->errors = array();
                dol_syslog(__METHOD__ . " Error: " . $this->error, LOG_ERR);
                return -1;
            }

            $idx = 0;
            $date_start = time(); // seconds
            $table_choices = array();
            foreach ($grouped_companies_to_check as $companies_to_check) {
                // Get companies infos from sirene by siret
                $siret_to_check = array_keys($companies_to_check);
                $result = $sirene->searchCompanies('', '', $siret_to_check, '', '', '', $nb_request_by_group);
                if ($result < 0) {
                    dol_syslog(__METHOD__ . " Error: " . $langs->trans('SireneErrorWhileGetCompaniesBySiret', implode(', ', $siret_to_check)) . ': ' . $sirene->errorsToString(), LOG_ERR);
                    $output .= '<span style="color: red;">' . $langs->trans('SireneErrorWhileGetCompaniesBySiret', implode(', ', $siret_to_check)) . '<br>' . $sirene->errorsToString('<br>') . '</span>' . "<br>";
                    $error++;
                } elseif (!empty($sirene->companies_results)) {
                    $now = dol_now();
                    foreach ($sirene->companies_results as $company_infos) {
                        $company_sql_infos = $companies_to_check[$company_infos['siret']];

                        $company = new Societe($this->db);
                        $company->fetch($company_sql_infos->rowid);
                        $company->oldcopy = clone $company;

                        // Check if has properties to update
                        $modified = false;
                        $need_check = false;
                        foreach ($correspondance as $sirene_key => $property_name) {
                            $siren_value = trim($company_infos[$sirene_key]);
                            $property_value = $company->$property_name;

                            if ($siren_value != $property_value && $to_update[$property_name]) {
                                $company->$property_name = $siren_value;
                                $modified = true;
                            }
                        }

                        $this->db->begin();
                        $sub_error = 0;

                        if ($company_infos['status'] == self::COMPANY_ADMIN_STATUS_CLOSED) {
                            //tiers fermé
                            $table_choice = '<p>' . $langs->trans('CompanySireneMail') . ' : ' . $company_infos['company_name_all'] . '</p>';
                            $table_choice .= '<p>' . $langs->trans('AdressSireneMail') . ' : ' . $company_infos['address_all'] . '</p>';
                            $table_choice .= '<p>' . $langs->trans('DateCreationSireneMail') . ' : ' . $company_infos['date_creation'] . '</p>';
                            $table_choice .= '<p>' . $langs->trans('StatusSireneMail') . ' : ' . $company_infos['status_all'] . '</p>';
                            $table_choice .= '<p>' . $langs->trans('CodenafSireneMail') . ' : ' . $company_infos['codenaf_all'] . '</p>';
                            $table_choice .= '<p>' . $langs->trans('SirenSireneMail') . ' : ' . $company_infos['siren'] . '</p>';
                            $table_choice .= '<p>' . $langs->trans('SiretSireneMail') . ' : ' . $company_infos['siret'] . '</p>' . "\n";
                            $table_choice .= '<p>' . $langs->trans('MailLinkClosedCompany') . ' : </p><br />';
                            $table_choice .= '<li><a href="' . dol_buildpath('/societe/card.php', 2) . '?socid=' . $company_sql_infos->rowid . '">' . $company_infos['company_name_all'] . '</a></li>';//$langs->trans('SireneMessageClosed')

                            //$company->status = 0; // not close now, it's too dangerous
                            $company->array_options['options_sirene_company_admin_status'] = self::COMPANY_ADMIN_STATUS_CLOSED;
                            $company->array_options['options_sirene_status'] = 1;
                            $result = $company->updateExtraField('sirene_company_admin_status');
                            if ($result > 0) $result = $company->updateExtraField('sirene_status');
                            if ($result < 0) {
                                $error_msg = $langs->trans('SireneErrorWhileUpdateSireneStatus', $company_sql_infos->name, $company_sql_infos->rowid) . ': ' . $company->errorsToString();
                                dol_syslog(__METHOD__ . " Error: " . $error_msg, LOG_ERR);
                                $output .= '<span style="color: red;">' . $error_msg . '</span>' . "<br>";
                                $error++;
                                $sub_error++;
                            } else {
                                // add to table_choices[] only if the extrafields have been updated
                                // otherwise the same societe are being spammed in the mail listing the societe closed every 10 minutes
                                // we still get an error message with what failed on the card of the cron task
                                $table_choices[] = $table_choice;
                            }
                        }
                        // Has properties to update
                        if ($modified) {
                            if ($need_check) {
                                $company->array_options['options_sirene_status'] = 1;
                            }
                            $result = $company->update(0);
                            if ($result < 0 && $company->errors[0] !== "ErrorBadCustomerCodeSyntax" &&
                            $company->errors[0] !== "ErrorBadSupplierCodeSyntax" &&
                            $company->errors[0] !== "ErrorCustomerCodeRequired" &&
                            $company->errors[0] !== "ErrorSupplierCodeRequired") {
                                $error_msg = $langs->trans('SireneErrorWhileUpdateExtraFields', $company_sql_infos->name, $company_sql_infos->rowid) . ': ' . $company->errorsToString();
                                dol_syslog(__METHOD__ . " Error: " . $error_msg, LOG_ERR);
                                $output .= '<span style="color: red;">' . $error_msg . '</span>' . "<br>";
                                $error++;
                                $sub_error++;
                            } elseif ($result < 0) {
                                $company->array_options['options_sirene_status'] = 1;
                                $result = $company->updateExtraField('sirene_status');
                                $error_msg = $langs->trans('SireneErrorWhileUpdateExtraFields', $company_sql_infos->name, $company_sql_infos->rowid) . ': ' . $company->errorsToString();
                                dol_syslog(__METHOD__ . " Warning: " . $error_msg, LOG_WARNING);
                                $output .= '<span style="color: orange;">' . $error_msg . '</span>' . "<br>";
                            }
                        }

                        $company->array_options['options_sirene_cron_date'] = $now;
                        $result = $company->updateExtraField('sirene_cron_date');
                            if ($result < 0) {
                                $error_msg = $langs->trans('SireneErrorWhileUpdateExtraFields', $company_sql_infos->name, $company_sql_infos->rowid) . ': ' . $company->errorsToString();
                                dol_syslog(__METHOD__ . " Error: " . $error_msg, LOG_ERR);
                                $output .= '<span style="color: red;">' . $error_msg . '</span>' . "<br>";
                                $error++;
                                $sub_error++;
                            }

                        if ($sub_error) {
                            $this->db->rollback();
                        } else {
                            $this->db->commit();
                        }
                    }
                }

                // Sleep when more request than x by y second
                $idx++;
                $elapsed_time = time() - $date_start;
                if ($elapsed_time > $time_limit_all_request) {
                    $idx = 0;
                    $date_start = time(); // seconds
                } elseif ($idx >= $nb_request_by_time_limit && $elapsed_time <= $time_limit_all_request) {
                    sleep($time_limit_all_request - $elapsed_time);
                    $idx = 0;
                    $date_start = time(); // seconds
                }
            }
        }

        // envoi du mail que si au moins une societe a ferme
        if (!empty($conf->global->SIRENE_MAIL_TO_SEND) && !empty($table_choices)) {
            // Init to avoid errors
            $filepath = array();
            $filename = array();
            $mimetype = array();

            // Send email to assigned user
            $subject = '[' . $conf->global->MAIN_INFO_SOCIETE_NOM . '] ' . $langs->transnoentities('SireneMailCompaniesClosed');
            $message = '<p>' . $langs->trans('SireneMailCompaniesClosedBody') . '</p>' . "\n";

            //ALERTE SI TIERS FERMEE
            $message .= implode('<br /><br />', $table_choices);

            $mainstmpid = (!empty($conf->global->MAIN_MAIL_SMTPS_ID_EMAILING) ? $conf->global->MAIN_MAIL_SMTPS_ID_EMAILING : '');
            $sendto = dol_escape_htmltag($conf->global->SIRENE_MAIL_TO_SEND);
            $from = dolGetFirstLastname($user->firstname, $user->lastname) . '<' . $conf->global->MAIN_MAIL_EMAIL_FROM . '>';// $user->email

            $message = dol_nl2br($message);

            include_once DOL_DOCUMENT_ROOT . '/core/class/CMailFile.class.php';
            $mailfile = new CMailFile($subject, $sendto, $from, $message, $filepath, $mimetype, $filename, '', '', 0, -1);
            if ($mailfile->error) {
                setEventMessages($mailfile->error, $mailfile->errors, 'errors');
            } else {
                $result = $mailfile->sendfile();
                if ($result < 0 || !$result) {
                    $errors = array();
                    $langs->load("other");
                    if ($mailfile->error) {
                        $errors[] = $langs->trans('ErrorFailedToSendMail', $from, $sendto);
                        $errors[] = $mailfile->error;
                    } else {
                        $errors[] = ' No mail sent. Feature is disabled by option MAIN_DISABLE_ALL_MAILS';
                    }
                    $output .= '<span style="color: red;">' . join('<br>', $errors) . '</span>' . "<br>";
                    dol_syslog(__METHOD__ . " - Errors when send mail file : " . join('; ', $errors));
                    $error++;
                } else {
                    $output .= $langs->trans("MailSuccessfulySent", $from, $sendto) . "<br>";
                }
            }
        }

        if ($error) {
            $this->error = $output;
            $this->errors = array();
            return -1;
        } else {
            $output .= $langs->trans('SireneCheckInfosSuccess');
        }

        $this->error = "";
        $this->errors = array();
        $this->output = $output;
        $this->result = array("commandbackuplastdone" => "", "commandbackuptorun" => "");

        return 0;
    }


    /**
     * Create config array for Guzzle Client
     * 
     * @return array
     */
    private function getConnectionConfig()
    {
        global $conf;

        $proxy = [];
        $timeout = $conf->global->SIRENE_API_TIMEOUT;
        $connection_timeout = "";

        dol_syslog(__METHOD__, LOG_DEBUG);

        if (!empty($conf->global->MAIN_PROXY_USE) && !empty($conf->global->MAIN_PROXY_HOST)) {
            $host = $conf->global->MAIN_PROXY_HOST;
            
            if (strpos($host, 'http') !== false) {
                $split_uri = explode('://', $host);
                $protocol = $split_uri[0];
                $uri = $split_uri[1];
            } else {
                $protocol = 'https';
                $uri = $host;
            }

            $proxy_str = $protocol . '://' .$uri; 

            if (!empty($conf->global->MAIN_PROXY_USER) && !empty($conf->global->MAIN_PROXY_PASS)) {
                $login_data = $conf->global->MAIN_PROXY_USER . ':'. $conf->global->MAIN_PROXY_PASS;
                $proxy_str = $protocol . '://' . $login_data . '@' .$uri; 
            } 

            if (!empty($conf->global->MAIN_PROXY_PORT)) {
                $proxy_str = $proxy_str. ':' . $conf->global->MAIN_PROXY_PORT;
            }
            
            $proxy = ['https' => $proxy_str, 'http' => $proxy_str];

            if (! empty($conf->global->MAIN_USE_RESPONSE_TIMEOUT)) {
                $timeout = $conf->global->MAIN_USE_RESPONSE_TIMEOUT;
            }

            if (! empty($conf->global->MAIN_USE_CONNECT_TIMEOUT)) {
                $connection_timeout = $conf->global->MAIN_USE_CONNECT_TIMEOUT;
            }
        }
    
        return [
            'proxy' => $proxy,
            'timeout'  => $timeout,
            'connection_timeout' => isset($connection_timeout) ? $connection_timeout : ''
        ];
    }


    /**
     *  Connect to the sirene API
     *
     * @return    int                        <0 if KO, >0 if OK
     */
    public function connectionSireneApi()
    {
        global $conf;
        dol_syslog(__METHOD__, LOG_DEBUG);
        $this->errors = array();

        $connection_config = $this->getConnectionConfig();
        $connection_config['base_uri'] = $this->getSireneApiUrl();

        try {
            $this->client = new Client($connection_config);
        } catch (Exception $e) {
            $this->errors[] = $e->getMessage();
            dol_syslog(__METHOD__ . " Error: " . $e, LOG_ERR);
            return -1;
        }

        return 1;
    }

    /**
     *  Connect to the RNA API
     *
     * @return    int                        <0 if KO, >0 if OK
     */
    public function connectionRnaAPi()
    {
        global $conf;
        dol_syslog(__METHOD__, LOG_DEBUG);
        $this->errors = array();

        $connection_config = $this->getConnectionConfig();
        $connection_config['base_uri'] = 'https://siva-integ1.cegedim.cloud/apim/api-asso/api/structure/';

        try {
            $this->client = new Client($connection_config);
        } catch (Exception $e) {
            $this->errors[] = $e->getMessage();
            dol_syslog(__METHOD__ . " Error: " . $e, LOG_ERR);
            return -1;
        }

        return 1;
    }


    /**
     * Method to output saved errors
     *
     * @param string $separator Separator between each error
     * @return    string                        String with errors
     */
    public function errorsToString($separator = ', ')
    {
        return (is_array($this->errors) ? join($separator, $this->errors) : '');
    }


    /**
     * get a single company by its siret with  the sirene API
     * @param string $siret
     * @return  int <0 if KO, >0 if OK
     */
    public function getOneCompanyBySiret($siret)
    {
        global $conf;

        try {
            $response = $this->client->get('siret/' . $siret, [
                'headers'                              => $this->_getAuthorizationHeader(),
                'verify' => false
            ]);

            $result = json_decode($response->getBody()->getContents(), true);
            $company_infos = isset($result['etablissement']) ? $result['etablissement'] : array();
            $single_company = $this->createCompanyArrayFromSireneApi($company_infos);

            $this->companies_results[] =  $single_company;

            $_SESSION['companies_results'] = $this->companies_results;
            return 1;
        } catch (RequestException $e) {
            $response = $e->getResponse();

            if (isset($response)) {
                if ($response->getStatusCode() == 404)
                {
                    return 1;
                }
            }

            $this->handleRequestError($e);
            return -1;
        } catch (Exception $e) {

            $this->handleErrorAndWriteLog($e);
            return -1;
        }
    }

    /**
     *  Get all companies found when search with the sirene API
     * @param string        $company_name       Search by company name
     * @param string        $siren              Search by siren
     * @param string|array  $siret              Search by siret or a list of siret
     * @param string        $rna                Search by rna
     * @param string        $naf                Search by naf
     * @param string        $zipcode            Search by zip code
     * @param string        $nombre             Maximum number of results
     * @param string        $town             	Search by town
     * @param int           $only_open          Search only open
     * @param int           $only_company_siege Search by company siege only
     * @return  int                             <0 if KO, >0 if OK
     */
    public function searchCompanies($company_name, $siren, $siret, $rna, $naf, $zipcode, $nombre, $town='', $only_open = 0, $only_company_siege = 0)
    {
        global $conf, $langs;

        dol_include_once('/sirene/lib/sirene.lib.php');
        $langs->load('sirene@sirene');
        $siret = is_array($siret) ? $siret : (!empty($siret) ? array($siret) : array());

        $this->companies_results = array();
        $filter = array();
        if ($company_name !== '') $filter[] = 'raisonSociale:"' . str_replace('"', '\\"', $company_name) . '"';
        if ($siren !== '') $filter[] = 'siren:' . $siren;
        if (!empty($siret)) $filter[] = '(siret:' . implode(' OR siret:', $siret) . ')';
        if ($naf !== '') $filter[] = 'activitePrincipaleUniteLegale:' . $naf;
        if ($zipcode !== '') $filter[] = 'codePostalEtablissement:' . $zipcode;
        if ($town !== '') $filter[] = 'libelleCommuneEtablissement:"' . str_replace('"', '\\"', $town) . '"';
        if ($only_open) $filter[] = 'periode(etatAdministratifEtablissement:A)';
        if ($only_company_siege) $filter[] = 'etablissementSiege:T';
        if ($rna) $filter[] = 'identifiantAssociationUniteLegale:' . $rna;


        try {
            $response = $this->client->post('siret', [
                'headers'                              => $this->_getAuthorizationHeader(),
                GuzzleHttp\RequestOptions::FORM_PARAMS => ['q' => implode(' AND ', $filter), 'nombre' => $nombre, 'date' => dol_print_date(dol_now(), "%Y-%m-%d")],
                'verify' => false
            ]);

            $results = json_decode($response->getBody()->getContents(), true);
            $results = isset($results['etablissements']) ? $results['etablissements'] : array();
            foreach ($results as $company_infos) {
                $this->companies_results[] = $this->createCompanyArrayFromSireneApi($company_infos);
            }
            $_SESSION['companies_results'] = $this->companies_results;
            return 1;
        } catch (RequestException $e) {
            $response = $e->getResponse();

            if (isset($response)) {
                if ($response->getStatusCode() == 404) return 1;
            }

            $this->handleRequestError($e);

            return -1;
        } catch (Exception $e) {

            $this->handleErrorAndWriteLog($e);
            return -1;
        }
    }


    /**
     * @param $rna
     * get company info from the RNA api
     * 	@return  int  <0 if KO, >0 if OK
     */
    public function searchCompanyByRna($rna)
    {
        try {
            $response = $this->client->get($rna, [
                'verify' => false
            ]);

            $company_infos = json_decode($response->getBody()->getContents(), true);
            $company = $this->createCompanyArrayFromRnaApi($company_infos);

            if(isset($company_infos['erreur']))
            {
                // erreur 404
                return 1;
            }

            $this->companies_results[] = $company;

            $_SESSION['companies_results'] = $this->companies_results;

            return 1;
        } catch (RequestException $e) {
            $response = $e->getResponse();

            if (isset($response)) {
                if ($response->getStatusCode() == 404)
                {
                    return 1;
                }
            }

            $this->handleRequestError($e);

            return -1;
        } catch (Exception $e) {

            $this->handleErrorAndWriteLog($e);
            return -1;
        }
    }

    /**
     * @param array $company_infos from the RNA API
     * @return array $company
     */
    private function createCompanyArrayFromRnaApi($company_infos)
    {
        global $langs;

        if(isset($company_infos['id_rna']))
        {
            $rna = $company_infos['id_rna'];
        }

        if(isset($company_infos['id_siren']))
        {
            $siren = $company_infos['id_siren'];
        }

        if(isset($company_infos['identite']))
        {
            $company_identity = $company_infos['identite'];


            if(isset($company_identity['nom']))
            {
                $company_name_all = $company_identity['nom'];
                $company_name = $company_identity['nom'];
            }

            if(isset($company_identity['id_siret_siege']))
            {
                $siret = $company_identity['id_siret_siege'];
            }

            if(isset($company_identity['date_creation']))
            {
                $date_creation = $company_identity['date_creation'];
            }
            elseif(isset($company_identity['date_creat']))
            {
                $date_creation = $company_identity['date_creat'];
            }

            if(isset($company_identity['id_forme_juridique']))
            {
                $legalcategory = $company_identity['legalcategory'];

                $legalcategory = strval($legalcategory);
                $legalcategory = substr($legalcategory, 0, 2);
                $legalcategory = intval($legalcategory);

                $company['legalcategory_id'] = dol_getIdFromCode($this->db, $legalcategory, 'c_forme_juridique', 'code', 'libelle', 0);
                $legalcategory_id = dol_getIdFromCode($this->db, $legalcategory, 'c_forme_juridique', 'code', 'libelle', 0);
            }

            if($company_identity["active"] == true)
            {
                $status = self::COMPANY_ADMIN_STATUS_OPENED;
            }
            else
            {
                $status  =  self::COMPANY_ADMIN_STATUS_CLOSED;
            }
        }

        if(isset($company_infos['coordonnees']))
        {
            if(isset($company_infos['coordonnees']['adresse_siege_sirene']))
            {
                $company_address = $company_infos["coordonnees"]["adresse_siege_sirene"];
            }
            else
            {
                $company_address = $company_infos["coordonnees"]["adresse_siege"];
            }
        }

        if(isset($company_address))
        {

            $address_all = "";
            $address_all .=  ' ' .$company_address['num_voie'];
            $address_all .= ' ' . $company_address['type_voie'];
            $address_all .= ' ' . $company_address['voie'];

            $address = $address_all;

            $town = $company_address['commune'];

            $address_all .=  ' ' .$company_address['commune'];
            $address_all .= ' ' . $company_address['cp'] . ' ';

            $zip_code = $company_address['cp'];

            $state_all = $company_address['cp'];

            $state_san = preg_replace('[\W]', '', $state_all);
            $state_san = substr(str_pad($state_san, 5, "0", STR_PAD_LEFT), 0, 3);

            if ($state_san < 970) $state_san = substr($state_san, 0, 2); // Compatibilité DOM-TOM

            $state_id = dol_getIdFromCode($this->db, $state_san, 'c_departements', 'code_departement', 'rowid', 0);
        }

        $status_all = $status == self::COMPANY_ADMIN_STATUS_OPENED ? '<span style="color:green;">' . $langs->trans('SireneEstablishmentOpened') . '</span>' : '<span style="color:red;">' . $langs->trans('SireneEstablishmentClosed') . '</span>';

        return array(
            'company_name'         => (isset($company_name) ? $company_name : ''),
            'firstname'            => (isset($firstname) ? $firstname : ''),
            'civility'             => (isset($civility) ? $civility : ''),
            'company_name_alias'   => (isset($company_name_alias) ? $company_name_alias : ''),
            'private'              => (isset($private) ? $private : ''),
            'company_name_all'     => (isset($company_name_all) ? $company_name_all : ''),
            'date_creation'        => (isset($date_creation) ? $date_creation : ''),
            'status'               => (isset($status) ? $status : ''),
            'status_all'           => (isset($status_all) ? $status_all : ''),
            'address'              => (isset($address) ? $address : ''),
            'zipcode'              => (isset($zip_code) ? $zip_code : ''),
            'town'                 => (isset($town) ? $town : ''),
            'state'                => (isset($state) ? $state : ''),
            'state_id'             => (isset($state_id) ? $state_id : ''),
            'state_san'            => (isset($state_san) ? $state_san : ''),
            'state_all'            => (isset($state_all) ? $state_all : ''),
            'address_all'          => (isset($address_all) ? $address_all : ''),
            'siren'                => (isset($siren) ? $siren : ''),
            'siret'                => (isset($siret) ? $siret : ''),
            'rna'				   => (isset($rna) ? $rna : ''),
            'codenaf'              => (isset($codenaf) ? $codenaf : ''),
            'codenaf_san'          => (isset($codenaf_san) ? $codenaf_san : ''),
            'codenaf_all'          => (isset($codenaf_all) ? $codenaf_all : ''),
            'country'              => (isset($country) ? $country : ''),
            'country_san'          => (isset($country_san) ? $country_san : ''),
            'country_id'           => (isset($country_id) ? $country_id : ''),
            'country_code'         => (isset($country_code) ? $country_code : ''),
            'sirene_tva_intra'     => (isset($tva_intra) ? $tva_intra : ''),
            'staff_code'           => (isset($staff_code) ? $staff_code : ''),
            'staff'                => (isset($staff) ? $staff : ''),
            'staff_label'          => (isset($staff_label) ? $staff_label : ''),
            'legalcategory'        => (isset($legalcategory) ? $legalcategory : ''),
            'legalcategory_id'     => (isset($legalcategory_id) ? $legalcategory_id : ''),
            'siege'                => (isset($siege) ? $siege : ''),
        );
    }


    /**
     * @param array $company_infos from the Sirene API
     * @return array $company
     */
    private function createCompanyArrayFromSireneApi(array $company_infos)
    {
        global $langs, $conf;
        //$dict_id = sirene_codenaf_dict_id();

        $private = empty($company_infos['uniteLegale']['denominationUniteLegale']) && !empty($company_infos['uniteLegale']['nomUsageUniteLegale']) ? 1 : 0;

        if (!$private) {
            // Morale
            $company_name = $company_infos['uniteLegale']['nomUniteLegale'];
            if (empty($company_name)) $company_name = $company_infos['uniteLegale']['denominationUniteLegale'];

            $company_name_alias = '';
            if (empty($company_name_alias)) $company_name_alias = $company_infos['uniteLegale']['denominationUsuelle1UniteLegale'];
            if (empty($company_name_alias)) $company_name_alias = $company_infos['uniteLegale']['denominationUsuelle2UniteLegale'];
            if (empty($company_name_alias)) $company_name_alias = $company_infos['uniteLegale']['denominationUsuelle3UniteLegale'];
            if (empty($company_name_alias) && isset($company_infos['periodesEtablissement'][0])) {
                $etablissement_infos = $company_infos['periodesEtablissement'][0];
                $company_name_alias = $etablissement_infos['denominationUsuelleEtablissement'];
                if (empty($company_name_alias)) $company_name_alias = $etablissement_infos['enseigne1Etablissement'];
                if (empty($company_name_alias)) $company_name_alias = $etablissement_infos['enseigne2Etablissement'];
                if (empty($company_name_alias)) $company_name_alias = $etablissement_infos['enseigne3Etablissement'];
            }
            if(!empty($companies_name_alias)) {
                $company_name_alias = trim($company_name_alias);
            }
            if ($company_name == $company_name_alias) $company_name_alias = '';

            $company_name_all = $company_name;
        } else {
            // Physique
            $company_name = $company_infos['uniteLegale']['nomUsageUniteLegale'];

            $company_name_alias = '';
            $firstname = $company_infos['uniteLegale']['prenomUsuelUniteLegale'];
            if (empty($firstname)) $firstname = $company_infos['uniteLegale']['prenom1UniteLegale']; elseif ($firstname != $company_infos['uniteLegale']['prenom1UniteLegale']) $company_name_alias .= ' ' . $company_infos['uniteLegale']['prenom1UniteLegale'];
            if (empty($firstname)) $firstname = $company_infos['uniteLegale']['prenom2UniteLegale']; elseif ($firstname != $company_infos['uniteLegale']['prenom2UniteLegale']) $company_name_alias .= ' ' . $company_infos['uniteLegale']['prenom2UniteLegale'];
            if (empty($firstname)) $firstname = $company_infos['uniteLegale']['prenom3UniteLegale']; elseif ($firstname != $company_infos['uniteLegale']['prenom3UniteLegale']) $company_name_alias .= ' ' . $company_infos['uniteLegale']['prenom3UniteLegale'];
            if (empty($firstname)) $firstname = $company_infos['uniteLegale']['prenom4UniteLegale']; elseif ($firstname != $company_infos['uniteLegale']['prenom4UniteLegale']) $company_name_alias .= ' ' . $company_infos['uniteLegale']['prenom4UniteLegale'];
            if(!empty($companies_name_alias)) {
                $company_name_alias = trim($company_name_alias);
            }

            $civility = $company_infos['uniteLegale']['sexeUniteLegale'] == 'M' ? 'MR' : 'MME';
            $civility_all = $company_infos['uniteLegale']['sexeUniteLegale'] == 'M' ? 'M.' : 'Mme';

            $company_name_all = $civility_all . ' ' . dolGetFirstLastname($firstname, $company_name);
        }
            if($company_infos['uniteLegale']['identifiantAssociationUniteLegale']) {
                $rna = $company_infos['uniteLegale']['identifiantAssociationUniteLegale'];
            }
            if (!empty($company_name_alias)) $company_name_all .= ' (' . $company_name_alias . ')';

            $date_creation = $company_infos['dateCreationEtablissement'];
            if (!empty($date_creation)) {
                $date_tmp = strtotime($date_creation);
                $date_creation = dol_print_date($date_tmp, 'day');
            } else {
                $date_creation = 'N/A';
            }
            $status = '';
            $status_all = '';
            $firstPeriodeEtablissementInfo = array_values($company_infos['periodesEtablissement']);
            if (!empty($firstPeriodeEtablissementInfo)) {
                $firstPeriodeEtablissementInfo = $firstPeriodeEtablissementInfo[0];
                $status = $firstPeriodeEtablissementInfo['etatAdministratifEtablissement'];

                if ($status != self::COMPANY_ADMIN_STATUS_OPENED) {
                    $status_all = '<span style="color:red;">' . $langs->trans('SireneEstablishmentClosed') . '</span>';
                } else {
                    $status_all = '<span style="color:green;">' . $langs->trans('SireneEstablishmentOpened') . '</span>';
                }
            }

            $ifDefined = function($value) {
                return ((isset($value) && $value != '[ND]') ? $value : '');
            };

            $address_all = $ifDefined($company_infos['adresseEtablissement']['numeroVoieEtablissement']);
            $address_all .= $ifDefined($company_infos['adresseEtablissement']['indiceRepetitionEtablissement']);
            $address_all .= ' ' . $ifDefined($company_infos['adresseEtablissement']['typeVoieEtablissement']);
            $address_all .= ' ' . $ifDefined($company_infos['adresseEtablissement']['libelleVoieEtablissement']);
            $address = $address_all;
            $address_all .= ' ' . $ifDefined($company_infos['adresseEtablissement']['codePostalEtablissement']);
            $zip_code = $ifDefined($company_infos['adresseEtablissement']['codePostalEtablissement']);
            $address_all .= ' ' . $ifDefined($company_infos['adresseEtablissement']['libelleCommuneEtablissement']);
            $town = $ifDefined($company_infos['adresseEtablissement']['libelleCommuneEtablissement']);
            $address_all .= ' ' . $ifDefined($company_infos['adresseEtablissement']['libelleCommuneEtrangerEtablissement']);
            $address_all .= ' ' . $ifDefined($company_infos['adresseEtablissement']['distributionSpecialeEtablissement']);
            $address_all .= ' ' . $ifDefined($company_infos['adresseEtablissement']['libelleCedexEtablissement']);
            $address_all .= ' ' . $ifDefined($company_infos['adresseEtablissement']['libellePaysEtrangerEtablissement']);

            $siren = $company_infos['siren'];
            $siret = $company_infos['siret'];

            $latitude = $company_infos["adresseEtablissement"]["coordonneeLambertAbscisseEtablissement"];
            $longitude = $company_infos["adresseEtablissement"]["coordonneeLambertOrdonneeEtablissement"];

            $codenaf_all = $codenaf = $firstPeriodeEtablissementInfo['activitePrincipaleEtablissement'];
            $codenaf_san = preg_replace('[\W]', '', $codenaf);
            $codenaf_san = str_pad($codenaf_san, 5, "0", STR_PAD_LEFT);

            //effectif
            $staff_code = $company_infos['uniteLegale']['trancheEffectifsUniteLegale']; //recup code sirene pour effectif

            $staff_code = intval($staff_code);
            $staff_code = dol_getIdFromCode($this->db, $staff_code, 'c_sirene_staff', 'code_sirene_staff', 'code_dolibarr_staff', 0);
            $staff_label = dol_getIdFromCode($this->db, $staff_code, 'c_effectif', 'code', 'libelle', 0);
            $staff = dol_getIdFromCode($this->db, $staff_code, 'c_effectif', 'code', 'id', 0);

            // Forme juridique niveau 3 récupéré depuis l'API Sirene
            // Ne prendre que les 2 premiers chiffres pour s'aligner sur le niveau 2 qu'utilise Dolibarr dans la table llx_c_forme_juridique
            $legalcategory = $company_infos['uniteLegale']['categorieJuridiqueUniteLegale'];
            $legalcategory = strval($legalcategory);
            $legalcategory = substr($legalcategory, 0, 2);
            $legalcategory = intval($legalcategory);
            $legalcategory_id = dol_getIdFromCode($this->db, $legalcategory, 'c_forme_juridique', 'code', 'libelle', 0);

            // Siege
            $siege = 0;
            if (!empty($company_infos['etablissementSiege']) && $company_infos['etablissementSiege'] == 'true') {
                $siege = 1;
            }

            $sql = "SELECT label FROM " . MAIN_DB_PREFIX . "c_codenaf WHERE code = '" . $this->db->escape($codenaf_san) . "'";
            $resql = $this->db->query($sql);
            if ($resql) {
                if ($obj = $this->db->fetch_object($resql)) {
                    $codenaf_all .= ' - ' . $obj->label;
                } else {
                    $codenaf_all .= ' - ' . $langs->trans('CodeNafNotFound', dol_buildpath('/admin/dict.php', 1) . '?id=' . $dict_id);
                }
            }

            $state_all = $state = $company_infos['adresseEtablissement']['codePostalEtablissement'];
            $state_san = preg_replace('[\W]', '', $company_infos['adresseEtablissement']['codePostalEtablissement']);
            $state_san = substr(str_pad($state_san, 5, "0", STR_PAD_LEFT), 0, 3);
            if ($state_san < 970) $state_san = substr($state_san, 0, 2); // Compatibilité DOM-TOM
            $state_id = dol_getIdFromCode($this->db, $state_san, 'c_departements', 'code_departement', 'rowid', 0);
            $country_san = $company_infos['adresseEtablissement']['codePaysEtrangerEtablissement'];

            if (empty($country_san)) {
                $country_san = -1;
            }
            $country_code = dol_getIdFromCode($this->db, $country_san, 'c_sirene_country', 'code_sirene', 'country_code', 0);
            if ($country_code !== -1) {
                require_once DOL_DOCUMENT_ROOT . '/core/lib/company.lib.php';
                $country = getCountry($country_code);
                $country_id = dol_getIdFromCode($this->db, $country_code, 'c_sirene_country', 'country_code', 'rowid', 0);
            } else {
                $country_code = '';
                $country = $langs->trans('Unknown') . ' - ID: ' . $country_san . " - Name: " . $company_infos['adresseEtablissement']['libellePaysEtrangerEtablissement'];
                $country_id = 0;
            }

            $tva_intra = '';

            if ($country_code == 'FR') {
                // intra-community vat number calculation
                $coef = 97;
                $vatintracalc = fmod($company_infos['siren'], $coef);
                $vatintracalc2 = fmod((12 + 3 * $vatintracalc), $coef);
                $tva_intra = 'FR' . str_pad($vatintracalc2, 2, 0, STR_PAD_LEFT) . $company_infos['siren'];
            }

            return array(
                'company_name'         => $ifDefined($company_name),
                'firstname'            => $ifDefined($firstname),
                'civility'             => $ifDefined($civility),
                'company_name_alias'   => $ifDefined($company_name_alias),
                'private'              => $ifDefined($private),
                'company_name_all'     => $ifDefined($company_name_all),
                'date_creation'        => $ifDefined($date_creation),
                'status'               => $ifDefined($status),
                'status_all'           => $ifDefined($status_all),
                'address'              => $ifDefined($address),
                'zipcode'              => $ifDefined($zip_code),
                'town'                 => $ifDefined($town),
                'state'                => $ifDefined($state),
                'state_id'             => $ifDefined($state_id),
                'state_san'            => $ifDefined($state_san),
                'state_all'            => $ifDefined($state_all),
                'address_all'          => $ifDefined($address_all),
                'siren'                => $ifDefined($siren),
                'siret'                => $ifDefined($siret),
                'rna'				   => $ifDefined($rna),
                'codenaf'              => $ifDefined($codenaf),
                'codenaf_san'          => $ifDefined($codenaf_san),
                'codenaf_all'          => $ifDefined($codenaf_all),
                'country'              => $ifDefined($country),
                'country_san'          => $ifDefined($country_san),
                'country_id'           => $ifDefined($country_id),
                'country_code'         => $ifDefined($country_code),
                'sirene_tva_intra'     => $ifDefined($tva_intra),
                'staff_code'           => $ifDefined($staff_code),
                'staff'                => $ifDefined($staff),
                'staff_label'          => $ifDefined($staff_label),
                'legalcategory'        => $ifDefined($legalcategory),
                'legalcategory_id'     => $ifDefined($legalcategory_id),
                'latitude'     		   => $ifDefined($latitude),
                'longitude'     	   => $ifDefined($longitude),
                'siege'                => $siege,
            );
    }


    /**
     * set $this->errors[] when Exception is catch
     * @param Exception $e
     * @return void
     */
    private function handleErrorAndWriteLog(Exception $e)
    {
        global $conf;

        if (!empty($conf->global->SIRENE_DEBUG)) {
            $this->errors[] = (string) $e;
        } else {
            $this->errors[] = $e->getMessage();
        }

        dol_syslog(__METHOD__ . " Error: " . $e, LOG_ERR);
    }


    /** set $this->errors[] when RequestException is catch
     * @param RequestException $e
     * @return void
     */
    private function handleRequestError(RequestException $e)
    {
        global $conf, $langs;

            $request = $e->getRequest();
            $response = $e->getResponse();

            if (isset($response)) {

                if (!empty($conf->global->SIRENE_DEBUG)) {
                    if (isset($request)) $this->errors[] = $this->requestToString($request);
                    if (isset($response)) $this->errors[] = $this->responseToString($response);
                    else $this->errors[] = '<pre>' . dol_nl2br((string) $e) . '</pre>';
                } else {
                    $status_code = $response->getStatusCode();
                    $suggested_solution = $this->suggestSolutionToHTTPError($status_code);

                    $this->errors[] = (empty($suggested_solution) ? '' : '<div class="bold warning">'.$suggested_solution.'</div>').
                        '<br><b class="bold">' . $langs->trans('SireneResponseCode') . ': </b>' . $response->getStatusCode() .
                        '<br><b class="bold">' . $langs->trans('SireneResponseReasonPhrase') . ': </b>' . $response->getReasonPhrase() . '<br>';
                }
            } else {
                if (!empty($conf->global->SIRENE_DEBUG)) {
                    $this->errors[] = (string) $e;
                } else {
                    $this->errors[] = $e->getMessage();
                }
            }

            dol_syslog(__METHOD__ . " Error: " . dol_htmlentitiesbr_decode($this->errorsToString('<br>')), LOG_ERR);
    }


    /**
     *  Format the request to a string
     *
     * @param RequestInterface $request Request handler
     * @return    string                            Formatted string of the request
     */
    protected function requestToString(RequestInterface $request)
    {
        global $langs;

        $out = '';
        $out .= '<b class=bold>' . $langs->trans('SireneRequestData') . ': </b><br><hr>';
        $out .= '<b class=bold>' . $langs->trans('SireneRequestProtocolVersion') . ': </b>' . $request->getProtocolVersion() . '<br>';
        $out .= '<b class=bold>' . $langs->trans('SireneRequestUri') . ': </b>' . $request->getUri() . '<br>';
        $out .= '<b class=bold>' . $langs->trans('SireneRequestTarget') . ': </b>' . $request->getRequestTarget() . '<br>';
        $out .= '<b class=bold>' . $langs->trans('SireneRequestMethod') . ': </b>' . $request->getMethod() . '<br>';
        $out .= '<b class=bold>' . $langs->trans('SireneRequestHeaders') . ':</b><ul>';
        foreach ($request->getHeaders() as $name => $values) {
            $out .= '<li><b>' . $name . ': </b>' . implode(', ', $values) . '</li>';
        }
        $out .= '</ul>';
        $out .= '<b class=bold>' . $langs->trans('SireneRequestBody') . ': </b>';
        $out .= '<br><em>' . $request->getBody() . '</em><br>';
        return $out;
    }


    /**
     *  Format the response to a string
     *
     * @param ResponseInterface $response Response handler
     * @return    string                            Formatted string of the response
     */
    protected function responseToString(ResponseInterface $response)
    {
        global $langs;

        // Set useful help
        $status_code = $response->getStatusCode();
        $suggested_solution = $this->suggestSolutionToHTTPError($status_code);

        $out = '';
        $out .= '<b class=bold>' . $langs->trans('SireneResponseData') . ': </b><br>';
        $out .= '<hr>';
        $out .=  empty($suggested_solution) ? '' : '<div class="bold warning">'.$suggested_solution.'</div>';
        $out .= '<b class=bold>' . $langs->trans('SireneResponseProtocolVersion') . ': </b>' . $response->getProtocolVersion() . '<br>';
        $out .= '<b class=bold>' . $langs->trans('SireneResponseCode') . ': </b>' . $status_code . '<br>';
        $out .= '<b class=bold>' . $langs->trans('SireneResponseReasonPhrase') . ': </b>' . $response->getReasonPhrase().'<br>';
        $out .= '<b class=bold>' . $langs->trans('SireneResponseHeaders') . ':</b><ul>';
        foreach ($response->getHeaders() as $name => $values) {
            $out .= '<li><b>' . $name . ': </b>' . implode(', ', $values) . '</li>';
        }
        $out .= '</ul>';
        $out .= '<b class=bold>' . $langs->trans('SireneResponseBody') . ': </b>';
        $body = json_decode($response->getBody(), true);
        if (is_array($body)) {
            $out .= '<ul>';
            foreach ($body as $name => $values) {
                $out .= '<li><b>' . $name . ': </b>' . (is_array($values) || is_object($values) ? json_encode($values) : $values) . '</li>';
            }
            $out .= '</ul>';
        } else {
            $out .= '<br><em>' . $response->getBody() . '</em><br>';
        }
        return $out;
    }

    /**
     * Suggest a solution for the error.
     * Mainly for codes 401 and 500.
     * @param int	$err_code		error code
     * @return 	string		Translated string for a solution
     */
    protected function suggestSolutionToHTTPError(int $err_code)
    {
        global $langs;
        switch ($err_code) {
            case 401:
                $message = 'SireneSolutionForError401';
                break;
            case 500:
                $message = 'SireneSolutionForError500';
                break;
            default:
                $message = '';
        }
        return $langs->transnoentities($message);
    }
}
